// Quad.h : interface of the Quad class
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _QUAD_H
#define _QUAD_H

#include <iostream.h>

typedef          long  LONG;	// 32 Bit
typedef unsigned long ULONG;	// 32 Bit

/////////////////////////////////////////////////////////////////////////////
// Quad class

class Quad
{
	ULONG lo, hi;

	friend void Quad_Add(const Quad a, const Quad b , Quad& x);
	friend void Quad_Sub(const Quad a, const Quad b , Quad& x);
	friend void Quad_Mul(const Quad a, const Quad b , Quad& x);
	friend void Quad_Div(const Quad a, const Quad b , Quad& x, Quad& rem);

	friend void Quad_Shl(const Quad a, const ULONG l, Quad& x);
	friend void Quad_Shr(const Quad a, const ULONG l, Quad& x);

public:
	Quad();
	Quad(const Quad& q);
	Quad(const ULONG uLo, const ULONG uHi = 0);

	~Quad();

	operator ULONG() const;

	Quad& operator= (const Quad q);

	Quad operator+ (const Quad q) const;
	Quad operator- (const Quad q) const;
	Quad operator* (const Quad q) const;
	Quad operator/ (const Quad q) const;
	Quad operator% (const Quad q) const;

	Quad  operator* (const ULONG l) const;
	Quad  operator/ (const ULONG l) const;
	ULONG operator% (const ULONG l) const;

	Quad& operator+= (const Quad q);
	Quad& operator-= (const Quad q);
	Quad& operator*= (const Quad q);
	Quad& operator/= (const Quad q);
	Quad& operator%= (const Quad q);

	Quad& operator*= (const ULONG l);
	Quad& operator/= (const ULONG l);
	Quad& operator%= (const ULONG l);

	int operator== (const Quad q) const;
	int operator!= (const Quad q) const;
	int operator<  (const Quad q) const;
	int operator<= (const Quad q) const;
	int operator>  (const Quad q) const;
	int operator>= (const Quad q) const;

	friend ostream& operator<<(ostream& os, const Quad d);

	void FromTime(
		const ULONG day,
		const ULONG month,  // set to 0 for relative time
		const ULONG year,   // set to 0 for relative time
		const ULONG hour,
		const ULONG minute,
		const ULONG second,
		const ULONG millisecond
	);

	void ToAbsTime(
		ULONG& day,
		ULONG& month,
		ULONG& year,
		ULONG& hour,
		ULONG& minute,
		ULONG& second,
		ULONG& millisecond
	) const;
	void ToRelTime(
		ULONG& days,
		ULONG& hours,
		ULONG& minutes,
		ULONG& seconds,
		ULONG& milliseconds
	) const;

	// Just to have multiplication also the other way around
	friend Quad operator*(const ULONG l, const Quad q);
};

/////////////////////////////////////////////////////////////////////////////
// Inline member and friend functions

#ifndef DEBUG
#	define QUAD_INLINE inline
#endif

QUAD_INLINE Quad::Quad() : hi(0), lo(0)
	{}
QUAD_INLINE Quad::Quad(const Quad& q) : hi(q.hi), lo(q.lo)
	{}
QUAD_INLINE Quad::Quad(const ULONG uLo, const ULONG uHi) : hi(uHi), lo(uLo)
	{}
QUAD_INLINE Quad::~Quad()
	{}

QUAD_INLINE Quad::operator ULONG() const
	{ if (hi != 0) throw "ULONG overflow"; return lo; }

QUAD_INLINE Quad& Quad::operator=(const Quad q)
	{ hi = q.hi; lo = q.lo; return *this; }

QUAD_INLINE int Quad::operator==(const Quad q) const
	{ return (hi == q.hi && lo == q.lo); }
QUAD_INLINE int Quad::operator!=(const Quad q) const
	{ return (hi != q.hi || lo != q.lo); }
QUAD_INLINE int Quad::operator<(const Quad q) const
	{ return(hi < q.hi || (hi == q.hi && lo < q.lo)); }
QUAD_INLINE int Quad::operator<=(const Quad q) const
	{ return(hi <= q.hi || (hi == q.hi && lo <= q.lo)); }
QUAD_INLINE int Quad::operator>(const Quad q) const
	{ return(hi > q.hi || (hi == q.hi && lo > q.lo)); }
QUAD_INLINE int Quad::operator>=(const Quad q) const
	{ return(hi >= q.hi || (hi == q.hi && lo >= q.lo)); }

QUAD_INLINE Quad operator*(const ULONG l, const Quad q)
	{ return (q * l); }

#ifdef QUAD_INLINE
#	undef QUAD_INLINE
#endif

#endif // _QUAD_H
