// QuadTime.h : interface of the QuadTime/QuadTimeDiff classes
//

#ifndef _QUADTIME_H
#define _QUADTIME_H

#include <iostream.h>

#include "Quad.h"

/////////////////////////////////////////////////////////////////////////////
// QuadTimeDiff class

class QuadTimeDiff
{
	Quad qTime;

	friend class QuadTime;

public:
	QuadTimeDiff();
	QuadTimeDiff(const QuadTimeDiff& d);
	QuadTimeDiff(const Quad& q);
	QuadTimeDiff(
		const ULONG days,
		const ULONG hours = 0,
		const ULONG minutes = 0,
		const ULONG seconds = 0,
		const ULONG milliseconds = 0
	);

	~QuadTimeDiff();

	QuadTimeDiff& operator=(const QuadTimeDiff d);

	QuadTimeDiff operator+(const QuadTimeDiff d) const;
	QuadTimeDiff operator-(const QuadTimeDiff d) const;

	QuadTimeDiff& operator+=(const QuadTimeDiff d);
	QuadTimeDiff& operator-=(const QuadTimeDiff d);

	Quad         operator/(const QuadTimeDiff d) const;
	QuadTimeDiff operator%(const QuadTimeDiff d) const;

	QuadTimeDiff operator*(const ULONG l) const;
	QuadTimeDiff operator/(const ULONG l) const;

	QuadTimeDiff& operator*=(const ULONG l);
	QuadTimeDiff& operator/=(const ULONG l);

	int operator==(const QuadTimeDiff d) const;
	int operator!=(const QuadTimeDiff d) const;
	int operator< (const QuadTimeDiff d) const;
	int operator<=(const QuadTimeDiff d) const;
	int operator> (const QuadTimeDiff d) const;
	int operator>=(const QuadTimeDiff d) const;

	friend ostream& operator<<(ostream& os, const QuadTimeDiff d);

	void Split(
		ULONG& days,
		ULONG& hours,
		ULONG& minutes,
		ULONG& seconds,
		ULONG& milliseconds
	) const;

	// Just to have multiplication also the other way around
	friend QuadTimeDiff operator*(const ULONG l, const QuadTimeDiff d);
};

/////////////////////////////////////////////////////////////////////////////
// QuadTime class

class QuadTime
{
	Quad qTime;

	friend class QuadTimeDiff;

public:
	QuadTime();
	QuadTime(const QuadTime& t);
	QuadTime(const Quad& q);
	QuadTime(
		const ULONG day,
		const ULONG month,
		const ULONG year,
		const ULONG hour = 0,
		const ULONG minute = 0,
		const ULONG second = 0,
		const ULONG millisecond = 0
	);

	~QuadTime();

	QuadTime& operator=(const QuadTime t);

	QuadTime operator+(const QuadTimeDiff d) const;
	QuadTime operator-(const QuadTimeDiff d) const;

	QuadTime& operator+=(const QuadTimeDiff d);
	QuadTime& operator-=(const QuadTimeDiff d);

	QuadTimeDiff operator-(const QuadTime t) const;

	int operator==(const QuadTime t) const;
	int operator!=(const QuadTime t) const;
	int operator< (const QuadTime t) const;
	int operator<=(const QuadTime t) const;
	int operator> (const QuadTime t) const;
	int operator>=(const QuadTime t) const;

	friend ostream& operator<<(ostream& os, const QuadTime d);

	void SetNow(void);
	void SetToday(void);

	void Split(
		ULONG& day,
		ULONG& month,
		ULONG& year,
		ULONG& hour,
		ULONG& minute,
		ULONG& second,
		ULONG& millisecond
	) const;
};

/////////////////////////////////////////////////////////////////////////////
// Inline member functions

#ifndef DEBUG
#	define QT_INLINE inline
#endif

QT_INLINE QuadTimeDiff::QuadTimeDiff() : qTime(0,0)
	{}
QT_INLINE QuadTimeDiff::QuadTimeDiff(const QuadTimeDiff& d) : qTime(d.qTime)
	{}
QT_INLINE QuadTimeDiff::QuadTimeDiff(const Quad& q) : qTime(q)
	{}
QT_INLINE QuadTimeDiff::QuadTimeDiff(const ULONG days,const ULONG hours,const ULONG minutes,const ULONG seconds,const ULONG milliseconds)
	{ qTime.FromTime(days,0,0,hours,minutes,seconds,milliseconds); }
QT_INLINE QuadTimeDiff::~QuadTimeDiff()
	{}
QT_INLINE QuadTimeDiff& QuadTimeDiff::operator=(const QuadTimeDiff d)
	{ qTime = d.qTime; return *this; }
QT_INLINE QuadTimeDiff QuadTimeDiff::operator+(const QuadTimeDiff d) const
	{ return QuadTimeDiff(qTime + d.qTime); }
QT_INLINE QuadTimeDiff QuadTimeDiff::operator-(const QuadTimeDiff d) const
	{ return QuadTimeDiff(qTime - d.qTime); }
QT_INLINE QuadTimeDiff QuadTimeDiff::operator*(const ULONG l) const
	{ return QuadTimeDiff(qTime * l); }
QT_INLINE QuadTimeDiff QuadTimeDiff::operator/(const ULONG l) const
	{ return QuadTimeDiff(qTime / l); }
QT_INLINE Quad QuadTimeDiff::operator/(const QuadTimeDiff d) const
	{ return Quad(qTime / d.qTime); }
QT_INLINE QuadTimeDiff QuadTimeDiff::operator%(const QuadTimeDiff d) const
	{ return QuadTimeDiff(qTime % d.qTime); }
QT_INLINE QuadTimeDiff& QuadTimeDiff::operator+=(const QuadTimeDiff d)
	{ qTime += d.qTime; return *this; }
QT_INLINE QuadTimeDiff& QuadTimeDiff::operator-=(const QuadTimeDiff d)
	{ qTime -= d.qTime; return *this; }
QT_INLINE QuadTimeDiff& QuadTimeDiff::operator*=(const ULONG l)
	{ qTime *= l; return *this; }
QT_INLINE QuadTimeDiff& QuadTimeDiff::operator/=(const ULONG l)
	{ qTime /= l; return *this; }
QT_INLINE int QuadTimeDiff::operator==(const QuadTimeDiff d) const
	{ return (qTime == d.qTime); }
QT_INLINE int QuadTimeDiff::operator!=(const QuadTimeDiff d) const
	{ return (qTime != d.qTime); }
QT_INLINE int QuadTimeDiff::operator<(const QuadTimeDiff d) const
	{ return(qTime < d.qTime); }
QT_INLINE int QuadTimeDiff::operator<=(const QuadTimeDiff d) const
	{ return (qTime <= d.qTime); }
QT_INLINE int QuadTimeDiff::operator>(const QuadTimeDiff d) const
	{ return(qTime > d.qTime); }
QT_INLINE int QuadTimeDiff::operator>=(const QuadTimeDiff d) const
	{ return (qTime >= d.qTime); }
QT_INLINE void QuadTimeDiff::Split(ULONG& days,ULONG& hours,ULONG& minutes,ULONG& seconds,ULONG& milliseconds) const
	{ qTime.ToRelTime(days,hours,minutes,seconds,milliseconds); }

QT_INLINE QuadTimeDiff operator*(const ULONG l, const QuadTimeDiff d)
	{ return QuadTimeDiff(d.qTime * l); }

QT_INLINE QuadTime::QuadTime() : qTime(0,0)
	{}
QT_INLINE QuadTime::QuadTime(const QuadTime& t) : qTime(t.qTime)
	{}
QT_INLINE QuadTime::QuadTime(const Quad& q) : qTime(q)
	{}
QT_INLINE QuadTime::QuadTime(const ULONG day,const ULONG month,const ULONG year,const ULONG hour,const ULONG minute,const ULONG second,const ULONG millisecond)
	{ qTime.FromTime(day,month,year,hour,minute,second,millisecond); }
QT_INLINE QuadTime::~QuadTime()
	{}
QT_INLINE QuadTime& QuadTime::operator=(const QuadTime t)
	{ qTime = t.qTime; return *this; }
QT_INLINE QuadTime QuadTime::operator+(const QuadTimeDiff d) const
	{ return QuadTime(qTime + d.qTime); }
QT_INLINE QuadTime QuadTime::operator-(const QuadTimeDiff d) const
	{ return QuadTime(qTime - d.qTime); }
QT_INLINE QuadTime& QuadTime::operator+=(const QuadTimeDiff d)
	{ qTime += d.qTime; return *this; }
QT_INLINE QuadTime& QuadTime::operator-=(const QuadTimeDiff d)
	{ qTime -= d.qTime; return *this; }
QT_INLINE QuadTimeDiff QuadTime::operator-(const QuadTime t) const
	{ return QuadTimeDiff(qTime - t.qTime); }
QT_INLINE int QuadTime::operator==(const QuadTime t) const
	{ return (qTime == t.qTime); }
QT_INLINE int QuadTime::operator!=(const QuadTime t) const
	{ return (qTime != t.qTime); }
QT_INLINE int QuadTime::operator<(const QuadTime t) const
	{ return(qTime < t.qTime); }
QT_INLINE int QuadTime::operator<=(const QuadTime t) const
	{ return(qTime <= t.qTime); }
QT_INLINE int QuadTime::operator>(const QuadTime t) const
	{ return(qTime > t.qTime); }
QT_INLINE int QuadTime::operator>=(const QuadTime t) const
	{ return(qTime >= t.qTime); }
QT_INLINE void QuadTime::Split(ULONG& day,ULONG& month,ULONG& year,ULONG& hour,ULONG& minute,ULONG& second,ULONG& millisecond) const
	{ qTime.ToAbsTime(day,month,year,hour,minute,second,millisecond); }

#ifdef QT_INLINE
#	undef QT_INLINE
#endif

#endif // _QUADTIME_H
